/*****************************************************************
*B.LECLERC/Feelgood 07/2019                                      *
Thermometre et Barometre enregistreur de valeurs Mini et Maxi    *
Affichache des données immédiate et indication de la progression *
Mise en veille automatique et réactivation si changement         *
conséquent de l'une ou l'autre des mesures ou pour appui pour    *
consultation . Effacement possible pour relancer une nouvelle    *
période de surveillance                                          * 
*****************************************************************/

#include "BMP280.h" //Declaration de la sonde et de ses librairies
#define P0 1015.25   //Correction en fonction de l'altitude
#include <LiquidCrystal.h>  // Declaration de l'afficheur et de ses librairies
#include <EEPROM.h>  //Declaration de l'EEPROM et de ses librairies 
#include <avr/pgmspace.h>  //Organisation de l'espace mémoire
#define valid 8  //Declaration du bouton
#define LCD_LED  6 //Declaration du rétro-éclairage du LCD

int nVal = 0;  // Declaration de la variable du 'Menu'
int nTemp = 0;  // Declaration de la variable de 'boucle de tempo'

BMP280 bmp;
float Tmin,Tmax,Pmin,Pmax,Tprevious,Pprevious; //  Decleration des variables utilisees ...
byte progress = ' '; // Assignement du caractère fabrique .
byte Ptend;  // Sens de la tendence de la pression
LiquidCrystal lcd(12, 11, 5, 4, 3, 2);  //Connection du LCD : RS,E, D4-D7,RW>0V,



byte up[8] = {  // Fabrication caractere progression
  B00100,
  B01110,
  B11111,
  B00100,
  B00100,
  B00100,
  B00100,
};

byte down[8] = {  // Fabrication caractere regression
  B00100,
  B00100,
  B00100,
  B00100,
  B11111,
  B01110,
  B00100,
};

void setup()
{

  pinMode(valid, INPUT_PULLUP);  //Bouton en entrée tirée à "1"
  pinMode(LCD_LED, OUTPUT);  // Sortie de commande rétro-eclairage
  digitalWrite(LCD_LED, HIGH); // Allumage LCD
  
  EEPROM.get(5, Ptend); // Emplacement de stockage en mémoire EEPROM des données conservées
  EEPROM.get(10, Tmin);
  EEPROM.get(15, Tmax); 
  EEPROM.get(20, Pmin); 
  EEPROM.get(25, Pmax);
   
  if(!bmp.begin()){ // Démarage BMP260 
    while(1);       // Et tourne toujours ........

  }
  
  lcd.createChar(0, up); //Attribut le numéro 0 du caractère fabriqué "byte up[8]"
  lcd.createChar(1, down); //Attribut le numéro 1 du caractère fabriqué "byte down[8]"
  lcd.begin(16, 2); // Afficheur 2 lignes 16 caractères
  
  bmp.setOversampling(4);  // 0 à 4, les nombres les plus élevés correspondent aux sorties plus lentes et aux résolutions plus élevées

}


void loop()
{
  double T,P;  //température & pression
  char result = bmp.startMeasurment(); // début de mesure....
 
    if(result!=0){   //Si le résultat n'est pas nul ...
    delay(result);
    result = bmp.getTemperatureAndPressure(T,P);
            
     lcd.setCursor(0, 0);
     lcd.print("T = ");lcd.print(T - 2,1); lcd.print(" degC"); // affichage sur la 1°ligne, sur 1 décimale, avec la correction de l'écart de la sonde ...
     
      if (T > (Tprevious + 0.3)){ //Si la temperature monte de 0,3°...
         lcd.setCursor(14, 0);
         progress = byte(0);
         lcd.write(progress);   //Affiche la flèche monté sur la 14 ème case
         digitalWrite(LCD_LED, HIGH); // Allume le rétro éclairage 
         nTemp = 0;  // R à Z de la boucle de temporisation
         Tprevious = T;  //Réactualisation de la nouvelle valeur
         }
         
      if (T < (Tprevious - 0.3)){ //Si la temperature descend de 0,3°...
         lcd.setCursor(14, 0);
         progress = byte(1);
         lcd.write(progress);  //Affiche la flèche descente sur la 14 ème case
           digitalWrite(LCD_LED, HIGH); // Allume le rétro éclairage 
         nTemp = 0; // R à Z de la boucle de temporisation
         Tprevious = T;  //Réactualisation de la nouvelle valeur
         }
         
     lcd.setCursor(0, 1);
     lcd.print("P = ");lcd.print(P + 36,0); lcd.print(" hPa");  // affichage sur la 2°ligne avec la correction de l'écart de la sonde ... ...
     lcd.setCursor(14, 1);
     lcd.write(Ptend);  //Affiche la flèche mémorisée sur la 14 ème case 2ème ligne
              
       if (P > (Pprevious + 1)){ //Si la pression monte de 1 hPa  ...
         lcd.setCursor(14, 1);
          progress = byte(0);
          Ptend = progress;  //Memorisation de la progression de la pression
          EEPROM.put(5, Ptend); // dans la case 5
         lcd.write(progress);  //Affiche la flèche monté sur la 14 ème case 2ème ligne
           digitalWrite(LCD_LED, HIGH); // Allume le rétro éclairage 
         nTemp = 0;  // R à Z de la boucle de temporisation
         Pprevious = P;  //Réactualisation de la nouvelle valeur
         }
         
      if (P < (Pprevious - 1)){ //Si la pression descent de 1 hPa  ...
         lcd.setCursor(14, 1);
          progress = byte(1);
          Ptend = progress;
          EEPROM.put(5, Ptend); // dans la case 5
         lcd.write(progress);  //Affiche la flèche descente sur la 14 ème case 2ème ligne
           digitalWrite(LCD_LED, HIGH); // Allume le rétro éclairage 
         nTemp = 0;  // R à Z de la boucle de temporisation
         Pprevious = P;  //Réactualisation de la nouvelle valeur
         }
         
      if (T<Tmin) //a stocker en EEprom, si la valeur est inférieure à la précédente valeur mini
         {Tmin=T;  //Réactualisation de la nouvelle valeur
         EEPROM.put(10, Tmin); // dans la case 10
         }
      if (T>Tmax)  //a stocker en EEprom, si la valeur est superieure à la précédente valeur maxi
         {Tmax=T;  //Réactualisation de la nouvelle valeur
         EEPROM.put(15, Tmax);  // dans la case 15
         }
      if (P<Pmin) //a stocker en EEprom, si la valeur est inférieure à la précédente valeur mini
         {Pmin=P;  //Réactualisation de la nouvelle valeur
         EEPROM.put(20, Pmin);  // dans la case 20
         }
      if (P>Pmax)  //a stocker en EEprom, si la valeur est superieure à la précédente valeur maxi
         {Pmax=P;  //Réactualisation de la nouvelle valeur
         EEPROM.put(25, Pmax); // dans la case 25
         }
   }

     if (nTemp > 100){ digitalWrite(LCD_LED, LOW);}// Eteind le rétro-éclaraige sur la boucle de tempo est atteinte...
     else {digitalWrite(LCD_LED, HIGH); } // Sinon on maintien allumé...
nTemp++;  //Incrémentation de la boucle 
delay(200);
    

if (digitalRead(valid)==LOW) {  //Et si on appui sur le boutton ...
   nVal ++; // incrémentation du menu
     digitalWrite(LCD_LED, HIGH);  // Allume le rétro éclairage
     delay (5000); // 5" pour admirer l'affichage
     nTemp = 0;  // R à Z de la boucle de temporisation
     nAnalyse();  // saute au sous programme d'analyse des résultats 
    }

}



void nAnalyse() {
     lcd.clear(); //Effacement total de l'afficheur

  if (nVal== 1){  // Si le menu est égal à 1...
    lcd.print("Temp.Mini-Maxi");
        lcd.setCursor(0,1);
          lcd.print("Min:");lcd.print(Tmin-2,0);  // Affichage de la température mini-maxi, avec la correction de l'écart de la sonde ...
      lcd.print(" Max:");lcd.print(Tmax-2,0);
      nVal++;  // incrémentation du menu
    delay (5000);
    lcd.clear(); //Effacement total de l'afficheur
  }
  
  if (nVal== 2){  // Si le menu est égal à 2...
    lcd.print("Pres.Mini-Maxi");
      lcd.setCursor(0,1);
          lcd.print("Mi:");lcd.print(Pmin+36,0);  // Affichage de la pression mini-maxi, avec la correction de l'écart de la sonde ...
      lcd.print(" Ma:");lcd.print(Pmax+36,0);
      nVal++;  // incrémentation du menu
    delay (5000);
    lcd.clear(); //Effacement total de l'afficheur
  }
  
  if (nVal== 3){  // Si le menu est égal à 3...
    lcd.print("Reset Temp");  // Proposition de R à Z des températures mini-maxi 
        lcd.setCursor(0,1);
          lcd.print("Push to reset");
          delay (3000);
              if (digitalRead(valid) ==LOW) {  // si le bouton est maintenu pressé
                lcd.clear();
                Tmin = 20;
                Tmax = 20;
                EEPROM.put(10, Tmin);  // Et chargement d'une valeur moyenne 
                EEPROM.put(15, Tmax);
                lcd.print("Done !");  //Accusé d'effacement
                delay (3000);}
            nVal++;  // incrémentation du menu
        lcd.clear(); //Effacement total de l'afficheur      
   }
  
  if (nVal== 4){  // Si le menu est égal à 4...
    lcd.print("Reset Press");  // Proposition de R à Z des pressions mini-maxi
      lcd.setCursor(0,1);
          lcd.print("Push to reset");
          delay (3000);
              if (digitalRead(valid) ==LOW) {  // si le bouton est maintenu pressé
                lcd.clear();
                Pmin = 970;  // Et chargement d'une valeur moyenne 
                Pmax = 970;
                EEPROM.put(20, Pmin);
                EEPROM.put(25, Pmax);
                lcd.print("Done !");  //Accusé d'effacement
                delay (3000);}
           nVal++;  // incrémentation du menu
       lcd.clear(); //Effacement total de l'afficheur
   }
 
  if (nVal== 5){  // Si le menu est égal à 5...
      nVal = 0; // R à Z du menu
      lcd.clear(); //Effacement total de l'afficheur
    loop(); // retour au programme principal
  }
    
delay (500);
if (nVal != 0 ){ nAnalyse();}  //Retour au sous menu si la totalité n'a pas été passée 

  }
